#include<stdio.h>

/**
* Node class.
*
* @author       Taeho Oh
* @version      0.0.1
* @since        1999/06/01
*/
class Node
{
private:
	/**
	* The maximum number of the child node.
	*/
	const int maxChild = 10 ;

	/**
	* The type of the node.
	*/
	int type ;

	/**
	* The token of the node.
	*/
	int token ;

	/**
	* The value of the node.
	*/
	int value ;

	/**
	* The string of the node.
	*/
	char string[256] ;

	/**
	* The child of the node.
	*/
	Node *child[maxChild] ;

	/**
	* The sibling of the node.
	*/
	Node *sibling ;

	/**
	* The number of child.
	*/
	int numChild ;

	/**
	* It is 1, if the node has sibling ;
	*/
	int hasSibling ;
public:
	/**
	* Initialize node.
	*/
	Node(int type)
	{
		this->type = type ;
		token = 0 ;
		value = 0 ;
		numChild = 0 ;
		hasSibling = 0 ;
		for(int i=0; i < 256; i++)
			string[i] = '\0' ;
	}

	/**
	* Free allocated memory.
	*/
	~Node()
	{
		for(int i = 0; i < numChild; i++)
			if(child[i] != NULL)
				delete child[i] ;
		if(hasSibling == 1)
			delete sibling ;
	}

	/**
	* Set the type of the node.
	*
	* @param type type which will be set.
	* @return none.
	*/
	void setType(int type)
	{
		this->type = type ;
	}

	/**
	* Get the type of the node.
	*
	* @return the type of the node.
	*/
	int getType()
	{
		return type ;
	}

	/**
	* Set token of the node.
	*
	* @param token token which will be set.
	* @return none.
	*/
	void setToken(int token)
	{
		this->token = token ;
	}

	/**
	* Get token of the node.
	*
	* @return the token of the node.
	*/
	int getToken()
	{
		return token ;
	}

	/**
	* Set value of the node.
	*
	* @param val the new value.
	* @return none.
	*/
	void setValue(int value)
	{
		this->value = value ;
	}

	/**
	* Get the value of the node.
	*
	* @return the value of the node.
	*/
	int getValue()
	{
		return value ;
	}

	/**
	* Set the string of the node.
	*
	* @param string the new string.
	* @return none.
	*/
	void setString(char *string)
	{
		strcpy(this->string, string) ;
	}

	/**
	* Get the string of the node.
	*
	* @return the string of the node.
	*/
	char *getString()
	{
		return string ;
	}

	/**
	* Add a child node.
	*
	* @param child the child node which will be added.
	* @return none.
	*/
	void addChild(Node *child)
	{
		if(numChild < maxChild)
			this->child[numChild++] = child ;
		else
			printf("Child Exceeded\n") ;
	}

	/**
	* Get a child node.
	*
	* @param index the child node index.
	* @return the child node.
	*/
	Node *getChild(int index)
	{
		if(numChild <= index )
			return NULL ;
		return child[index] ;
	}

	/**
	* Add the sibling node.
	*
	* @param sibling the sibling node.
	* @return none.
	*/
	void addSibling(Node *sibling)
	{
		Node *newSibling ;

		for(newSibling = this; newSibling->hasSibling == 1; newSibling = newSibling->getSibling()) ;

		newSibling->sibling = sibling ;
		newSibling->hasSibling = 1 ;
	}

	/**
	* Get the sibling node.
	*
	* @return the sibling node.
	*/
	Node *getSibling()
	{
		if(hasSibling == 1)
			return sibling ;
		else
			return NULL ;
	}

	/**
	* Get the number of children.
	*
	* @return the number of children.
	*/
	int getNumChild()
	{
		return numChild ;
	}

	/**
	* Get the number of siblings.
	*
	* @return the number of siblings.
	*/
	int getNumSibling()
	{
		Node *numSibling ;
		int i = 1 ;

		for(numSibling = this; numSibling->hasSibling == 1; numSibling = numSibling->getSibling())
			i++ ;
		return i ;
	}
} ;
