
%{

#include<stdio.h>
#include<stdlib.h>
#include<string.h>
#include"Node.cc"
#include"SSymbolTable.cc"
#include"Emit.cc"
#include"Code.cc"
#include"TypeCheck.cc"

#define YYSTYPE Node *

int yylex() ;
int yyerror(char *s) ;
extern char *yytext ;

Node *rootNode ;

%}

%token IF ELSE WHILE VOID INT RETURN
%token ASSOP EQ NE LT GT LE GE
%token PLUS MINUS MULT DIVI

%token SPARENL SPARENR MPARENL MPARENR BPARENL BPARENR

%token COMMA SEMICLN
%token NUMBER
%token ID

%nonassoc LOWER_THAN_ELSE
%nonassoc ELSE

%%

program
	:	declarationlist
		{
			rootNode = $1 ;
		}
	;
declarationlist
	:	declarationlist declaration
		{
			if($1 != NULL)
			{
				$1->addSibling($2) ;
				$$ = $1 ;
			}
			else
				$$ = $2 ;
		}
	|	declaration
		{
			$$ = $1 ;
		}
	;
declaration
	:	vardeclaration
		{
			$$ = $1 ;
		}
	|	fundeclaration
		{
			$$ = $1 ;
		}
	;
vardeclaration
	:	typespecifier ident SEMICLN
		{
			$$ = new Node(4) ;
			$$->addChild($1) ;
			$$->addChild($2) ;
		}
	|	typespecifier ident BPARENL number BPARENR SEMICLN
		{
			$$ = new Node(4) ;
			$$->addChild($1) ;
			$$->addChild($2) ;
			$$->addChild($4) ;
		}
	;
typespecifier
	:	INT
		{
			$$ = new Node(30) ;
			$$->setToken(INT) ;
		}
	|	VOID
		{
			$$ = new Node(30) ;
			$$->setToken(VOID) ;
		}
	;
fundeclaration
	:	typespecifier ident SPARENL params SPARENR compoundstmt
		{
			$$ = new Node(6) ;
			$$->addChild($1) ;
			$$->addChild($2) ;
			$$->addChild($4) ;
			$$->addChild($6) ;
		}
	;
params
	:	paramlist
		{
			$$ = $1 ;
		}
	|	VOID
		{
			$$ = new Node(30) ;
			$$->setToken(VOID) ;
		}
	;
paramlist
	:	paramlist COMMA param
		{
			if($1 != NULL)
			{
				$1->addSibling($3) ;
				$$ = $1 ;
			}
			else
				$$ = $3 ;
		}
	|	param
		{
			$$ = $1 ;
		}
	;
param
	:	typespecifier ident
		{
			$$ = new Node(9) ;
			$$->addChild($1) ;
			$$->addChild($2) ;
		}
	|	typespecifier ident BPARENL BPARENR
		{
			$$ = new Node(9) ;
			$$->addChild($1) ;
			$$->addChild($2) ;
			Node *newNode ;
			newNode = new Node(30) ;
			$$->addChild(newNode) ;
		}
	;
compoundstmt
	:	MPARENL localdeclarations statementlist MPARENR
		{
			$$ = new Node(10) ;
			$$->addChild($2) ;
			$$->addChild($3) ;
		}
	;
localdeclarations
	:	localdeclarations vardeclaration
		{
			if($1 != NULL)
			{
				$1->addSibling($2) ;
				$$ = $1 ;
			}
			else
				$$ = $2 ;
		}
	|	empty
		{
			$$ = NULL ;
		}
	;
statementlist
	:	statementlist statement
		{
			if($1 != NULL)
			{
				$1->addSibling($2) ;
				$$ = $1 ;
			}
			else
				$$ = $2 ;
		}
	|	empty
		{
			$$ = NULL ;
		}
	;
statement
	:	expressionstmt
		{
			$$ = $1 ;
		}
	|	compoundstmt
		{
			$$ = $1 ;
		}
	|	selectionstmt
		{
			$$ = $1 ;
		}
	|	iterationstmt
		{
			$$ = $1 ;
		}
	|	returnstmt
		{
			$$ = $1 ;
		}
	;
expressionstmt
	:	expression SEMICLN
		{
			$$ = $1 ;
		}
	|	SEMICLN
	;
selectionstmt
	:	IF SPARENL expression SPARENR statement %prec LOWER_THAN_ELSE
		{
			$$ = new Node(15) ;
			$$->addChild($3) ;
			$$->addChild($5) ;
		}
	|	IF SPARENL expression SPARENR statement ELSE statement
		{
			$$ = new Node(15) ;
			$$->addChild($3) ;
			$$->addChild($5) ;
			$$->addChild($7) ;
		}
	;
iterationstmt
	:	WHILE SPARENL expression SPARENR statement
		{
			$$ = new Node(16) ;
			$$->addChild($3) ;
			$$->addChild($5) ;
		}
	;
returnstmt
	:	RETURN SEMICLN
		{
			$$ = new Node(17) ;
		}
	|	RETURN expression SEMICLN
		{
			$$ = new Node(17) ;
			$$->addChild($2) ;
		}
	;
expression
	:	var ASSOP expression
		{
			$$ = new Node(18) ;
			$$->setToken(ASSOP) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	simpleexpression
		{
			$$ = $1 ;
		}
	;
var
	:	ident
		{
			$$ = $1 ;
		}
	|	ident BPARENL expression BPARENR
		{
			$$ = new Node(31) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	;
simpleexpression
	:	additiveexpression LE additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(LE) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression LT additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(LT) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression GT additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(GT) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression GE additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(GE) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression EQ additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(EQ) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression NE additiveexpression
		{
			$$ = new Node(20) ;
			$$->setToken(NE) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression
		{
			$$ = $1 ;
		}
	;
additiveexpression
	:	additiveexpression PLUS term
		{
			$$ = new Node(22) ;
			$$->setToken(PLUS) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	additiveexpression MINUS term
		{
			$$ = new Node(22) ;
			$$->setToken(MINUS) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	term
		{
			$$ = $1 ;
		}
	;
term
	:	term MULT factor
		{
			$$ = new Node(24) ;
			$$->setToken(MULT) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	term DIVI factor
		{
			$$ = new Node(24) ;
			$$->setToken(DIVI) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	|	factor
		{
			$$ = $1 ;
		}
	;
factor
	:	SPARENL expression SPARENR
		{
			$$ = $2 ;
		}
	|	var
		{
			$$ = $1 ;
		}
	|	call
		{
			$$ = $1 ;
		}
	|	number
		{
			$$ = $1 ;
		}
	;
call
	:	ident SPARENL args SPARENR
		{
			$$ = new Node(27) ;
			$$->addChild($1) ;
			$$->addChild($3) ;
		}
	;
args
	:	arglist
		{
			$$ = $1 ;
		}
	|	empty
		{
			$$ = NULL ;
		}
	;
arglist
	:	arglist COMMA expression
		{
			if($1 != NULL)
			{
				$1->addSibling($3) ;
				$$ = $1 ;
			}
			else
				$$ = $3 ;
		}
	|	expression
		{
			$$ = $1 ;
		}
	;
ident
	:	ID
		{
			$$ = new Node(30) ;
			$$->setToken(ID) ;
			$$->setString(yytext) ;
		}
	;
number
	:	NUMBER
		{
			$$ = new Node(30) ;
			$$->setToken(NUMBER) ;
			$$->setValue(atoi(yytext)) ;
		}
	;
empty
	:
		{
			$$ = NULL ;
		}
	;
%%

main()
{
	yyparse() ;

	if(typeCheck(rootNode) == 0)
		exit(1) ;
	emitCode(LD, 6, 0, 0) ;
	emitCode(LD, 4, 0, 0) ;
	genCode(rootNode) ;
	emitCode(HALT, 0, 0, 0) ;
}

int yyerror(char *s)
{
	fprintf(stderr,"%s\n",s) ;
	return 1 ;
}

